{{
********************************************************
*   Phase modulation DEMO version 1.0                     *
*                                                      *
*               (C) 2007 Parallax, Inc.                *
*                                                      *
* This program uses the Propeller Demo Board, Rev C    *
* The microphone is digitized using Sigma Delta ADC on *
* CTRA and the samples are modulated over the desired  *
* synthesized frequency generated on CTRB.             *
*                                                      *
*       coded by Beau Schwabe (Parallax)               *
********************************************************
}}
CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000


' At 80MHz the ADC/DAC sample resolutions and rates are as follows:
'
' sample   sample               
' bits       rate               
' ----------------              
' 5       2.5 MHz               
' 6      1.25 MHz               
' 7       625 KHz               
' 8       313 KHz               
' 9       156 KHz               
' 10       78 KHz               
' 11       39 KHz               
' 12     19.5 KHz               
' 13     9.77 KHz               
' 14     4.88 KHz               
                                
  bits = 11                                   'try different values from table here

PUB go

  PinA := 8                                   'SigmaDelta ADC feedback pin
  PinB := 9                                   'SigmaDelta ADC drive pin
  PinC := 0                                   'Carrier frequency Pin

  Freq := 900_000                             'Carrier frequency


  Synth(PinC, Freq)                           'Setup and calculate FRQx and CTRx values 
  MaskPin_B  := |< PinB                       'Create Mask value for ADC drive pin
  MaskPin_C  := |< PinC                       'Create Mask value for Carrier frequency pin
  cognew(@asm_entry, 0)                       'launch assembly program into a COG

PUB Synth(Pin_, Freq_) | s, d
    Freq_ := Freq_ #> 0 <# 128_000_000        'limit frequency range
    if Freq_ < 500_000                        'if 0 to 499_999 Hz,
       CTRx := constant(%00100 << 26)         '..set NCO mode
       s := 1                                 '..shift = 1
    else                                      'if 500_000 to 128_000_000 Hz,
       CTRx := constant(%00010 << 26)         '..set PLL mode
       d := >|((Freq_ - 1) / 1_000_000)       'determine PLLDIV
       s := 4 - d                             'determine shift
       CTRx |= d << 23                        'set PLLDIV
    FRQx := fraction(Freq_, CLKFREQ, s)       'Compute FRQx value
    CTRx |= Pin_                              'set PINA to complete CTRx value

PRI fraction(a, b, shift) : f
    if shift > 0                              'if shift, pre-shift a or b left
       a <<= shift                            'to maintain significant bits while 
    if shift < 0                              'insuring proper result
       b <<= -shift
 
    repeat 32                                 'perform long division of a/b
      f <<= 1
      if a => b
         a -= b
         f++           
      a <<= 1  
DAT

'
' Assembly program
'
              org

asm_entry
'Setup Sigma Delta ADC on CTRA
              mov       dira,MaskPin_B        'make ADC drive pin an output
              movs      ctra,PinA             'Set ADC feedback IO Pin
              movd      ctra,PinB             'Set ADC drive IO Pin
              movi      ctra,#%01001_000      'Set CTRA mode ; POS W/FEEDBACK
              mov       frqa,#1               'Start Sigma Delta ADC

'Setup Frequency Synthesis on CTRB
              mov       ctrb,CTRx             'Set CTRB to pre-calculated value
              mov       frqb,FRQx             'Set FRQB to pre-calculated value
              or        dira,MaskPin_C        'make Carrier frequency pin an output
                                              'and Start Frequency Synthesizer.       

              mov       asm_cnt,cnt           'prepare for WAITCNT loop
              add       asm_cnt,asm_cycles
              
'Loop here forever and modulate the audio data over the Frequency Synthesizer.              
:loop         waitcnt   asm_cnt,asm_cycles    'wait for next CNT value (timing is determinant after WAITCNT)

              mov       asm_sample,phsa       'capture PHSA and get difference
              sub       asm_sample,asm_old
              add       asm_old,asm_sample
                                                         
              shl       asm_sample,#32-bits   'justify sample and output to PHSB
              add       phsb, asm_sample      'Here is where the Phase modulation takes place
                                            
              jmp       #:loop                'wait for next sample period
'
' Data
'
asm_cycles              long      |< bits - 1 'sample time

MaskPin_B               long    0
MaskPin_C               long    0
PinA                    long    0
PinB                    long    0
PinC                    long    0
Freq                    long    0
CTRx                    long    0
FRQx                    long    0

asm_cnt       res       1
asm_old       res       1
asm_sample    res       1
